require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/activity/activity_set_state'
require 'uu/os/activity/activity_attributes'
require 'uu/os/activity/activity_create'
require 'uu/os/activity/activity_set_attributes'
require 'uu/os/activity/activity_get_instance_list'
require 'uu/os/activity/activity_get_activity_list'
require 'uu/os/activity/activity_type'
require 'uu/os/activity/activity_state_type'
require 'uu/os/activity/activity_state_subtype'
require 'uu/os/activity/activity_set_executive_role'
require 'uu/os/activity/activity_set_competent_role'

# Module Unicorn Universe.
module UU
  # Module Operating System.
  module OS
    # Activity service.
    module Activity
      extend self

      # Service path
      PATH = 'ues/core/workflow/UESActivity'

      # This command sets a new state for the specified activity. You can enter the state template using ues uri or by specifying its type and, additionally, subtype.
      #
      # @param [String, UU::OS::UESURI] activity_uri UESURI of the activity. Must not be nil.
      # @param [UU::OS::Activity::ActivitySetState] activity Specific options.
      # @option activity [String, UU::OS::UESURI] :template_uri When nil is set, attribute type (and subtype) will be used to setting activity state.
      # @option activity [String] :type When nil is set, attribute templateUri must be set.
      # @option activity [String] :subtype When nil is set, the first possible subtype will be used.
      # @option activity [String] :comment Activity state's comment. When nil is set, comment will be left empty.
      #
      # @example
      #   # Set activity state by type and subtype.
      #   UU::OS::Activity.set_state('ues:TERRITORY:ARTIFACT:ACTIVITY', :type => 'ACTIVE', :subtype => 'II')
      #
      #   # Set activity state by ues uri of state template.
      #   UU::OS::Activity.set_state('ues:TERRITORY:ARTIFACT:ACTIVITY', :template_uri => 'ues:TERRITORY:META_ARTIFACT:ACTIVITY_TEMPLATE')
      def set_state(activity_uri, activity = nil)
        svc = UU::OS::REST::RemoteClient.new(Activity)
        payload = UU::OS::Activity::ActivitySetState.new(activity).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('setState', activity_uri, payload)
        end
      end
      
      # This command sets new competent role for the specified activity.
      #
      # @param [String, UU::OS::UESURI] activity_uri UESURI of the activity. Must not be nil.
      # @param [UU::OS::Activity::ActivitySetCompetentRole] activity DTO containing URI of new competent role. Cannot be nil.
      # @option activity [String, UU::OS::UESURI] :competent_role_uri URI of new competent role.
      #
      # @example
      #   # Set competent role.
      #   UU::OS::Activity.set_competent_role('ues:TERRITORY:ARTIFACT:ACTIVITY', :competent_role_uri => 'ues:UNI:UNI')
      def set_competent_role(activity_uri, activity = nil)
        svc = UU::OS::REST::RemoteClient.new(Activity)
        payload = UU::OS::Activity::ActivitySetCompetentRole.new(activity).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('setCompetentRole', activity_uri, payload)
        end
      end
      
      # This command sets new executive role for the specified activity.
      #
      # @param [String, UU::OS::UESURI] activity_uri UESURI of the activity. Must not be nil.
      # @param [UU::OS::Activity::ActivitySetExecutiveRole] activity DTO containing URI of new executive role. Cannot be nil.
      # @option activity [String, UU::OS::UESURI] :executive_role_uri URI of new executive role.
      #
      # @example
      #   # Set executive role.
      #   UU::OS::Activity.set_executive_role('ues:TERRITORY:ARTIFACT:ACTIVITY', :executive_role_uri => 'ues:UNI:UNI')
      def set_executive_role(activity_uri, activity = nil)
        svc = UU::OS::REST::RemoteClient.new(Activity)
        payload = UU::OS::Activity::ActivitySetExecutiveRole.new(activity).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('setExecutiveRole', activity_uri, payload)
        end
      end

      # This command returns attributes representing the activity specified by the activityUri
      # parameter. The command does not change the state of the activity in the system.
      #
      # @param [String, UU::OS::UESURI] activity_uri UESURI of an activity whose attributes are returned
      #
      # @return [ActivityAttributes] DTO containing the activity attributes
      #
      # @example
      #   # Get activity attributes.
      #   UU::OS::Activity.get_attributes('ues:TERRITORY:ARTIFACT:ACTIVITY')
      #
      #   # Get activity attributes with main entity normalized uri.
      #   UU::OS::Activity.get_attributes('ues:TERRITORY[412357]:NEWS[417204]:ACTIVITYATRIBUTES[57420895257493462]')
      #
      def get_attributes(activity_uri)
        svc = UU::OS::REST::RemoteClient.new(Activity)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', activity_uri)
          return Activity::ActivityAttributes.new(res)
        end
      end

      # This command creates a new activity on the specified artifact. At the very least
      # activity template or the activity type have to be specified in this command. <b>Attention:</b>
      # No activity actions or conditions are copied from the template when the new activity
      # is created by templateUri.
      #
      # @param [String, UU::OS::UESURI] parent_node_uri UESURI of an artifact (or a parent activity) where the new activity is created
      # @param [ActivityCreate] activity DTO containing attributes of a new activity
      # @option activity [String, UU::OS::UESURI] :template_uri UESURI of the activity template. The template determines the type of the new activity. When nil is set, the template is determined by the type attribute (which cannot be nil in that case).
      # @option activity [UU::OS::Activity::ActivityType] :activity_type Specifies the type of the new activity. When nil is set, the activity type is determined by the template (templateUri attribute, which cannot be nil in that case).
      # @option activity [String] :name Name of the new activity. When nil is set, the name is taken from the template tName. When the template does not define a tDescription, the create command fails.
      # @option activity [String] :code  Code of the new activity. When nil is set, the code is generated automatically using the system sequence. The code must be unique in a given artifact, otherwise the create command fails.
      # @option activity [String] :description Description of the new activity. When nil is set, the description is taken from the template tDescription. When the template does not define a tDescription, it is left empty.
      # @option activity [Object] :date_from Specifies the date when the activity is shown in the related users digital workspace. When nil is set, the actual date (00:00) is used for all types of activities except of an activity of type RESERVE_TIME or COVER_MEETING. In that case, parent's dateFrom is used. If it is nil, the actual time is used (but if the actual time is later than dateStart of the activity, dateStart is set as dateFrom instead). <br> Subactivity specifics: Always the dateFrom of the parent is used.
      # @option activity [Object] :date_start Specifies the start of the meeting. Cannot be nil for an activity of type RESERVE_TIME or COVER_MEETING, has to be nil for all other types of activities. Date must be in iso8601 format.
      # @option activity [Object] :date_to Specifies the date when the activity should be finished. When nil is set, the date is determined by the template settings. Cannot be nil for an activity of type RESERVE_TIME or COVER_MEETING. <br> Subactivity specifics: The date must greater than parent's dateFrom and lesser or equal as parent's dateTo. When it is not possible to resolve dateFrom of parent activity, dateTo must be nil. Date must be in iso8601 format.
      # @option activity [Object] :date_expiration Specifies the date when the activity is automatically finished. This parameter is an object and it can contain three possible types of values: If it's a Date, it sets the activity to expire at that date. Such a date has to be after the dateTo attribute. If it's set to false, the activity will never expire (expiration is forbidden). If it's nil, then the date is determined by the template settings. <br> Subactivity specifics: The date has to be less or equal to parent's dateExpiration. When it is not possible to resolve dateFrom of parent activity, dateExpiration must be nil. Date must be in iso8601 format.
      # @option activity [String, UU::OS::UESURI] :competent_role_uri The competent role for the activity. When nil is set, the result of authorization is used. <br> Subactivity specifics: Nil must be set at creation subactivity (the competent role is taken from parent activity).
      # @option activity [Array] :executive_role_uris The executive role or roles of the new activity. Role or group role could be used. When more than one role is specified or when a group role is used, the command creates a delegation for every role. <br> When nil is set, the same role as for competentRoleUri is used. <br> Subactivity specifics: When nil is set at creation subactivity, the command fails.
      # @option activity [String, UU::OS::UESURI] :related_artifact_uri Deprecated since UES7-08 use attribute related_object_uri. Related artifact of the new activity. When nil is set, the related artifact is taken from the template. When the template does not define it, it is left empty.
      # @option activity [String, UU::OS::UESURI] :related_object_uri Related object of the new activity. When nil is set, the related object is taken from the template. When the template does not define it, it is left empty.
      # @option activity [Boolean] :allow_executive_role_change Specifies if it will be possible to change an executive role of the new activity in the future. When nil is set, is taken from the template. Cannot be false for an activity of type RESERVE_TIME or COVER_MEETING.
      # @option activity [Boolean] :allow_subactivities Specifies if it will be possible to create new child activities on the new activity in the future. When nil is set, is taken from the template. Cannot be false for an activity of type RESERVE_TIME or COVER_MEETING.
      # @option activity [Boolean] :block_time_in_calendar Specifies if the calendar should block the time for the time of meeting. Only for an activity of type RESERVE_TIME or COVER_MEETING. When nil is set, is taken from the template. See {ActivityType}.
      # @option activity [Boolean] :is_competent_role_participating Specifies if the root activity should be created as a cover activity, in case child activites are created. When nil is set, it is true. Only for an activity of type RESERVE_TIME or COVER_MEETING. See {ActivityType}.
      # @option activity [Boolean] :notify Specifies whether users who are cast in executiveRoleUris will be notified of the new activity in their Digital Workspace and by e-mail (if they have enabled e-mail notifications in their personal settings). If nil is set, notifications will be enabled so the users will be notified.
      # @option activity [String] :parameters Parameters for a macro or other processing tool. Value is a UTF-8 string with the maximum size of 4000 bytes.
      #
      # @return [UU::OS::UESURI] UESURI of the new activity
      # @example
      #   # Create activity of type reservation.
      #   activityCreate= UU::OS::Activity::ActivityCreate.new({
      #   activity_type: UU::OS::Activity::ActivityType::TIME_RESERVATION,
      #   allow_executive_role_change:true,
      #   allow_subactivities:true,
      #   block_time_in_calendar:false,
      #   code:"ACTIVITYCREATETASK",
      #   competent_role_uri:"ues:UNI:UNI~DIR",
      #   date_start:"2013-09-03",
      #   date_to:"2015-09-04",
      #   description:"Some test description",
      #   executive_role_uris:["ues:UNI:US08"],
      #   name:"Test activity",
      #   notify:true,
      #   related_artifact_uri:"ues:UNI:NEWS",
      #   })
      #
      #   UU::OS::Activity.create('ues:UNI:NEWS',activityCreate)
      #
      #   # Create activity of type TASK
      #   activityCreate= UU::OS::Activity::ActivityCreate.new({
      #   activity_type: UU::OS::Activity::ActivityType::TASK,
      #   allow_executive_role_change:true,
      #   allow_subactivities:true,
      #   code:"ACTIVITYCREATETASK",
      #   competent_role_uri:"ues:UNI:UNI~DIR",
      #   date_from:"2013-09-03",
      #   description:"Some test description",
      #   executive_role_uris:["ues:UNI:US08"],
      #   name:"Test activity",
      #   notify:true,
      #   related_artifact_uri:"ues:UNI:NEWS",
      #   })
      #
      #   UU::OS::Activity.create('ues:UNI:NEWS',activityCreate)
      #
      def create(parent_node_uri, activity = nil)
        svc = UU::OS::REST::RemoteClient.new(Activity)
        payload = UU::OS::Activity::ActivityCreate.new(activity).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', parent_node_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # This command updates some basic attributes of an activity.
      #
      # @param [String, UU::OS::UESURI] activity_uri UESURI of the activity whose attributes are to be changed
      # @param [ActivitySetAttributes] activity DTO containing new attributes the activity
      # @option activity [String] :name The new name of the activity. If it is not specified, the value is not modified.
      # @option activity [String] :code The new code of the activity. If it is not specified, the value is not modified. The code must be unique in a given artifact, otherwise the setAttributes command fails.
      # @option activity [String] :description The new description of the activity. If it is not specified, the value is not modified.
      # @option activity [Object] :date_from Specifies the date when the activity is shown in the related users digital workspace. Date must be in iso8601 format.
      # @option activity [Object] :date_start The new start of the meeting. Could be specified only for activities of type <code>TIME_RESERVATION</code> or <code>COVER_MEETING</code>. If it is not specified, the value is not modified. <br> Subactivity specifics: Always the date must be nil. See {ActivityType}. Date must be in iso8601 format.
      # @option activity [Object] :date_to The new date when the activity should be finished. If it is not specified, the value is not modified. Date must be in iso8601 format.
      # @option activity [Object] :date_expiration Specifies the date when the activity is automatically finished. This parameter is an object and it can contain three possible types of values: If it's a Date, it sets the activity to expire at that date. Such a date has to be after the dateTo attribute. If it's set to false, the activity will never expire (expiration is forbidden). If it is not specified, the value is not modified. Subactivity specifics: The date has to be less or equal to parent's dateExpiration. When it is not possible to resolve dateFrom of parent activity, dateExpiration must be nil. Date must be in iso8601 format. If it is not specified, the value is not modified.
      # @option activity [String, UU::OS::UESURI] :related_artifact_uri The new related artifact of the activity. If it is not specified, the related artifact reference is not modified. If nil UESURI ("ues:{[-1]}:{[-1]}:") is set, the related artifact is set empty. Subactivity specifics: The date must greater than parent's dateFrom and lesser or equal as parent's dateTo. When it is not possible to resolve dateFrom of parent activity, dateTo must be nil.
      # @option activity [String, UU::OS::UESURI] :related_object_uri The new related object of the activity. If it is not specified, the related object reference is not modified. If nil UESURI ("ues:{[-1]}:{[-1]}:") is set, the related artifact is set empty.
      # @option activity [Boolean] :allow_executive_role_change Specifies if it will be possible to change an executive role of the activity in the future. If it is not specified, the value is not modified.
      # @option activity [Boolean] :allow_subactivities Specifies if it will be possible to create new child activities on the activity in the future. If it is not specified, the value is not modified.
      # @option activity [Boolean] :block_time_in_calendar Specifies if the calendar should block the time for the time of meeting. Only for activities type <code>TIME_RESERVATION</code> or <code>COVER_MEETING</code>. If it is not specified, the value is not modified. See {ActivityType}.
      # @option activity [String, UU::OS::UESURI] :competent_role_uri <b>Deprecated.</b> The new competent role for the activity. If it is not specified, the value is not modified.
      # @option activity [String, UU::OS::UESURI] :executive_role_uri <b>Deprecated.</b> The new executive role of the activity. If it is not specified, the value is not modified.<br> Subactivity specifics: nil must be set at creation subactivity (the competent role is taken from parent activity).
      # @option activity [Boolean] :notify Specifies whether users who are cast in executiveRoleUri will be notified of the modified activity in their Digital Workspace and by e-mail (if they have enabled e-mail notifications in their personal settings). If nil is set, notifications will be enabled so the users will be notified.
      # @option activity [String] :parameters Parameters for a macro or other processing tool. Value is a UTF-8 string with the maximum size of 4000 bytes. If it is not specified, the value is not modified.
      #
      # @return [UU::OS::UESURI] UESURI of the activity. It could be changed because command could change name and code.
      # 
      # @example
      #   # Set activity new expiration
      #   activitySetAttributes= UU::OS::Activity::ActivitySetAttributes.new({
      #   allow_executive_role_change:true,
      #   allow_subactivities:true, 
      #   date_expiration:"2013-10-03",     
      #   description:"Some test description",
      #   executiveRoleUri:"ues:UNI:US08",
      #   name:"New name",  
      #   })
      #
      #   activityUri = UU::OS::Activity.set_attributes('ues:UNI:ART:ACTIVITY',activitySetAttributes)
      #
      def set_attributes(activity_uri, activity = nil)
        svc = UU::OS::REST::RemoteClient.new(Activity)
        payload = UU::OS::Activity::ActivitySetAttributes.new(activity).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', activity_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # This command deletes an unfinished activity. Planned activity will be removed
      # from the system, activity in progress will be set to the canceled state.
      #
      # @param [String, UU::OS::UESURI] activity_uri UESURI of the activity which is about to be deleted.
      # 
      # @example
      #   # Command that deletes activity
      #   UU::OS::Activity.delete('ues:TERRITORY:ART:ACTIVITY')
      #
      def delete(activity_uri)
        svc = UU::OS::REST::RemoteClient.new(Activity)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('delete', activity_uri)
        end
      end

      # Returns list of all instances of the recurrent activity. Returned list does not contain any objects filtered out by executed UESQuery.
      #
      # @param [String, UU::OS::UESURI] activity_uri UESURI of the activity for which instances are to be listed.
      # @param criteria DTO containing the criteria for filtering of the result list. Criteria are defined using UESQuery. Allowed filtering parameters are name, code, activityType, allowExecutiveRoleChange, allowSubactivities, blockTimeInCalendar, activityStateType, activityStateSubtype, recurrent, termModifiable, dateFrom, dateStart, dateTo and dateFinished.
      # @option criteria [String] :query The UES query string. It has to conform the ues query syntax (see documentation for more info). Filtering and order of the result list can be specified there. When nil is set, no filtering is applied and entries are ordered by name and code as the second criteria.
      # @return [UU::OS::REST::ResultList<Activity::ActivityGetInstanceList, Activity::ActivityAttributes>] List of DTO representing activities.
      #
      # @example
      #   # Get activity instance list.
      #   getInstanceList = UU::OS::Activity.get_instance_list('ues:TER:ART:ACT',:query=>'ORDER BY activityType,code')
      #  
      def get_instance_list(activity_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(Activity)
        dto = Activity::ActivityGetInstanceList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getInstanceList', activity_uri)
          return UU::OS::REST::ResultList.new(Activity::ActivityGetInstanceList, Activity::ActivityAttributes, res)
        end
      end

      # Returns list of the activities and sub activities for the specified artifact or
      # activity. Returned list does not contain any objects filtered out by executed
      # UESQuery.
      #
      # @param [String, UU::OS::UESURI] parent_node_uri UES URI of the artifact or activity for which activities are about to be listed.
      # @param [ActivityGetActivityList] criteria DTO containing the criteria for filtering of the result list. Criteria are defined using UESQuery. Allowed filtering parameters are name, code, activityType, allowExecutiveRoleChange, allowSubactivities, blockTimeInCalendar, activityStateType, activityStateSubtype, recurrent, termModifiable, dateFrom, dateStart, dateTo and dateFinished.
      # @option criteria [String] :query The UES query string. It has to conform the ues query syntax (see documentation for more info). Filtering and order of the result list can be specified there. When nil is set, no filtering is applied and entries are ordered by name and code as the second criteria.
      # @option criteria [Boolean] :recursive Flag for recursive listing. It means that if the listed activities contains sub activities and the recursive flag is true, all sub activities are in result too. When the recursive flag is nil or false, only entries are returned exactly from the listed artifact without sub activities.
      #
      # @return [UU::OS::REST::ResultList<Activity::ActivityGetActivityList, Activity::ActivityAttributes>] List of DTO representing activities.
      # 
      # @example 
      #   # Command with recursive listing.
      #   UU::OS::Activity.get_activity_list('ues:TER:ART:ACT', :query=>'ORDER BY name DESC', :recursive=>true) 
      #
      def get_activity_list(parent_node_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(Activity)
        dto = Activity::ActivityGetActivityList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        svc.add_parameter('recursive', dto.recursive)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getActivityList', parent_node_uri)
          return UU::OS::REST::ResultList.new(Activity::ActivityGetActivityList, Activity::ActivityAttributes, res)
        end
      end

      #
      # Retrieves the list of set states on a given activity based on provided criteria.
      #
      # @param [String, UU::OS::UESURI] activity_uri UESURI of the activity (model or instance), whose communication will be retrieved.
      # @param [ActivityGetStateHistoryList] criteria Object containing paging criteria.
      # @return [UU::OS::REST::ResultList<Activity::ActivityGetStateHistoryList, Activity::ActivityStateAttributes>] Returns list of activity states attributes. Contains instance of ActivityStateAttributes.
      #
      # @example
      #   # UESURI of the activity
      #   activity_uri = 'ues:UNI-BT:44191582648611234:57420906578882155'
      #
      #   # Get all activity states that were set on the activity
      #   list = UU::OS::Activity.get_state_history_list(activity_uri)
      #
      def get_state_history_list(activity_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(Activity)
        dto = Activity::ActivityGetStateHistoryList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getStateHistoryList', activity_uri)
          return UU::OS::REST::ResultList.new(Activity::ActivityGetStateHistoryList, Activity::ActivityStateAttributes, res)
        end
      end
      
      #
      # Returns a list of activity state templates which can be set by the current user for the specified activity.
      #
      # @param [String, UU::OS::UESURI] activity_uri UESURI of the activity whose state templates will be returned for the current user.
      # @param [ActivityGetStateHistoryList] criteria The search criteria.
      # @option criteria [String] :query The query for filtering and ordering of the result list. It must conform the uesQuery syntax (see the documentation for more info). The result list can be filtered and ordered by the following attributes: name, code, type, entryState, exitState, order. If set to nil, no filtering will be applied and entries will be ordered by order and name.
      # @return [UU::OS::REST::ResultList<Activity::ActivityGetStateTemplateList, Activity::ActivityStateTemplateAttributes>] Returns list of activity states attributes. Contains instance of ActivityStateAttributes.
      #
      # @example
      #   # Returns a list of activity state templates which can be set by the current user for the specified activity which have the initial type
      #   UU::OS::Activity.get_state_template_list('ues:TERRITORY:ARTIFACT:ACTIVITY', :query => 'type=INITIAL')
      def get_state_template_list(activity_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(Activity)
        dto = Activity::ActivityGetStateTemplateList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getStateTemplateList', activity_uri)
          return UU::OS::REST::ResultList.new(Activity::ActivityGetStateTemplateList, Activity::ActivityStateTemplateAttributes, res)
        end
      end

    end
  end
end
