require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/all_from_organizational_unit_cast/all_from_organizational_unit_cast_attributes'
require 'uu/os/all_from_organizational_unit_cast/all_from_organizational_unit_cast_create'
require 'uu/os/all_from_organizational_unit_cast/all_from_organizational_unit_cast_delete'
require 'uu/os/all_from_organizational_unit_cast/all_from_organizational_unit_cast_exists'
require 'uu/os/all_from_organizational_unit_cast/all_from_organizational_unit_cast_get_attributes'
require 'uu/os/all_from_organizational_unit_cast/all_from_organizational_unit_cast_get_cast_list'
require 'uu/os/all_from_organizational_unit_cast/all_from_organizational_unit_cast_set_attributes'

module UU
  module OS
    # Module All From Organizational Unit Cast.
    # @deprecated All From OU will not be used in the new authorization (v4).
    module AllFromOrganizationalUnitCast
      extend self

      # Service path
      PATH = 'uu/os/AllFromOrganizationalUnitCast'

      # Casts a subject (specified in the parameter casted_subject_uri) to the group All From Organizational 
      # Unit which is derived from specified organizational unit. Only cast between subject and group All From 
      # Organizational Unit from the same organizational unit is supported. Any change of user authorization 
      # resulting from the cast modification will take effect approximately after 15 minutes.
      #
      # @param [String, UU::OS::UESURI] organizational_unit_uri UESURI of the organizational unit from which is the group All From Organization Unit derived. Another subject is casted to the derived group.
      # @param [AllFromOrganizationalUnitCastCreate] cast Object containing attributes of the cast
      # @option cast [String, UU::OS::UESURI] :casted_subject_uri UESURI of the subject (role or group) which is to be casted to the group All From Organizational Unit identified by the command main entity. Never nil.
      # @option cast [String, UU::OS::Cast::CastLevel] :level The level of the new cast relation. If nil is set EXECUTIVE level is used.
      # @option cast [String] :date_to The informative field describing the date when the cast should expire. If nil is set, no date is set.
      #
      # @example
      #   # Cast a role to All From OU of its organizational unit
      #   UU::OS::AllFromOrganizationalUnitCast.create('ues:TERRITORY:ORGANIZATIONAL_UNIT',
      #           :casted_subject_uri => 'ues:TERRITORY:ROLE',
      #           :level => 'ASSISTANCE'
      #   )
      #
      def create(organizational_unit_uri, cast = nil)
        svc = UU::OS::REST::RemoteClient.new(AllFromOrganizationalUnitCast)
        payload = UU::OS::AllFromOrganizationalUnitCast::AllFromOrganizationalUnitCastCreate.new(cast).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('create', organizational_unit_uri, payload)
        end
      end

      # Returns attributes of subject cast into the group All From Organizational Unit
      # which is derived from specified organizational unit.
      #
      # @param [String, UU::OS::UESURI] organizational_unit_uri UESURI of the organizational unit from which is the group All From Organization Unit derived.
      # @param [AllFromOrganizationalUnitCastGetAttributes] cast Object containing attributes of the cast
      # @option cast [String, UU::OS::UESURI] :casted_subject_uri UESURI of the subject (role or group) which is casted to the group All From Organizational Unit identified by the command main entity. Cannot be nil.
      #
      # @return [AllFromOrganizationalUnitCastAttributes] Object containing the cast attributes
      #
      # @example
      #   # Get attributes of a cast to All From OU
      #   attributes = UU::OS::AllFromOrganizationalUnitCast.get_attributes('ues:TERRITORY:ORGANIZATIONAL_UNIT',
      #           :casted_subject_uri => 'ues:TERRITORY:ROLE'
      #   )
      #
      def get_attributes(organizational_unit_uri, cast)
        svc = UU::OS::REST::RemoteClient.new(AllFromOrganizationalUnitCast)
        get_dto = UU::OS::AllFromOrganizationalUnitCast::AllFromOrganizationalUnitCastGetAttributes.new(cast)
        svc.add_parameter('castedSubjectUri', get_dto.casted_subject_uri)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', organizational_unit_uri)
          return AllFromOrganizationalUnitCast::AllFromOrganizationalUnitCastAttributes.new(res)
        end
      end

      # Returns list of casts according to the group All From Organizational Unit which is derived
      # from the specified organizational unit. The result list can be filtered and ordered by the following
      # attributes: level, dateFrom, state, castedSubjectName, castedSubjectCode, castedSubjectEntityTypeUri.
      # The list is sorted by the date of creation of casts (dateFrom) by default.
      #
      # @param [String, UU::OS::UESURI] organizational_unit_uri UESURI of the organizational unit from which is the group All From Organization Unit derived.
      # @param [AllFromOrganizationalUnitCastGetCastList] criteria Specific options.
      # @option criteria [String] :query Query for filtering and ordering of the result list (more information in {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY uuQuery - Guideline}). It is possible to filter and order the result list by castedSubjectName, castedSubjectCode, dateFrom, state, level and castedSubjectEntityTypeUri.
      #
      # @return [Array] An Array with the command result. Contains instances of {AllFromOrganizationalUnitCastAttributes}.
      #
      # @example
      #   # Returns list of casts which have executive level
      #   UU::OS::AllFromOrganizationalUnitCast.get_cast_list('ues:TERRITORY:ORGANIZATIONAL_UNIT', :query => 'level="EXECUTIVE"')
      #
      def get_cast_list(organizational_unit_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(AllFromOrganizationalUnitCast)
        dto = AllFromOrganizationalUnitCast::AllFromOrganizationalUnitCastGetCastList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getCastList', organizational_unit_uri)
          return UU::OS::REST::ResultList.new(AllFromOrganizationalUnitCast::AllFromOrganizationalUnitCastGetCastList, AllFromOrganizationalUnitCast::AllFromOrganizationalUnitCastAttributes, res)
        end
      end

      # Sets basic attributes of the subject cast into the All From Organizational Unit which is
      # derived from the specified organizational unit.
      #
      # @param [String, UU::OS::UESURI] organizational_unit_uri UESURI of the organizational unit from which is the group All From Organization Unit derived.
      # @param [AllFromOrganizationalUnitCastSetAttributes] cast Object containing attributes of the cast
      # @option cast [String, UU::OS::UESURI] :casted_subject_uri UESURI of the subject (role or group) which is casted to the group All From Organizational Unit identified by the command main entity. Cannot be nil.
      # @option cast [String, UU::OS::Cast::CastLevel] :level The new level of the cast relation. If nil is set the level is not changed.
      # @option cast [String] :date_to The informative field describing the date when the cast should expire. Note that there is no business logic behind this field at the moment, so it serves only as a note. If nil is set, the date is not changed.
      # @option cast [String, UU::OS::Cast::CastState] :state The new state of the cast. Only ACTIVE casts are involved in the business rules computations. If nil is set, the state is not changed.
      #
      # @example
      #   # Change cast level of a cast to All From OU
      #   UU::OS::AllFromOrganizationalUnitCast.set_attributes('ues:TERRITORY:ORGANIZATIONAL_UNIT',
      #           :casted_subject_uri => 'ues:TERRITORY:ROLE',
      #           :level => UU::OS::Cast::CastLevel::ASSISTANCE
      #   )
      #
      def set_attributes(organizational_unit_uri, cast = nil)
        svc = UU::OS::REST::RemoteClient.new(AllFromOrganizationalUnitCast)
        payload = UU::OS::AllFromOrganizationalUnitCast::AllFromOrganizationalUnitCastSetAttributes.new(cast).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('setAttributes', organizational_unit_uri, payload)
        end
      end

      # Deletes the cast from the system. Any change of user authorization resulting from the cast
      # modification will take effect approximately after 15 minutes.
      #
      # @param [String, UU::OS::UESURI] organizational_unit_uri UESURI of the organizational unit from which is the group All From Organization Unit derived.
      # @param [AllFromOrganizationalUnitCastDelete] cast Object containing UESURI of the casted subject.
      # @option cast [String, UU::OS::UESURI] :casted_subject_uri UESURI of the subject (role or group) which is casted to the group All From Organizational Unit identified by the command main entity. Cannot be nil.
      #
      # @example
      #   # Delete a cast to All From OU
      #   UU::OS::AllFromOrganizationalUnitCast.delete('ues:TERRITORY:ORGANIZATIONAL_UNIT', :casted_subject_uri => 'ues:TERRITORY:ROLE')
      #
      def delete(organizational_unit_uri, cast = nil)
        svc = UU::OS::REST::RemoteClient.new(AllFromOrganizationalUnitCast)
        payload = UU::OS::AllFromOrganizationalUnitCast::AllFromOrganizationalUnitCastDelete.new(cast).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('delete', organizational_unit_uri, payload)
        end
      end

      # Checks whether the subject (specified in the parameter casted_subject_uri) is casted
      # to the group All From Organizational Unit which is derived from the specified
      # organizational unit.
      #
      # @param [String, UU::OS::UESURI] organizational_unit_uri UESURI of the organizational unit from which is the group All From Organization Unit derived.
      # @param [AllFromOrganizationalUnitCastExists] cast Specific options.
      # @option cast [String, UU::OS::UESURI] :casted_subject_uri UESURI of the subject (role or group) which is casted to the group All From Organizational Unit identified by the command main entity. Cannot be nil.
      # @option cast [Boolean] :recursive Flag indicating whether the uuCommand checks indirect cast existence. In current uuCommand version it is possible to check indirect cast only for access roles. Default value is false.
      #
      # @return [Boolean] True if cast exists, else false.
      #
      # @example
      #   # Check direct cast existence (the role has cast to All From Organizational Unit derived from ORGANIZATIONAL_UNIT).
      #   UU::OS::AllFromOrganizationalUnitCast.exists('ues:TERRITORY:ORGANIZATIONAL_UNIT', :casted_subject_uri => 'ues:TERRITORY:ROLE')
      #
      #   # Check indirect access role cast existence (the access role has cast to a role which has cast to All From Organizational Unit derived from ORGANIZATIONAL_UNIT).
      #   UU::OS::AllFromOrganizationalUnitCast.exists('ues:TERRITORY:ORGANIZATIONAL_UNIT', :casted_subject_uri => 'ues:TERRITORY:ACCESS_ROLE', :recursive => true)
      #
      def exists(organizational_unit_uri, cast)
        svc = UU::OS::REST::RemoteClient.new(AllFromOrganizationalUnitCast)
        dto = UU::OS::AllFromOrganizationalUnitCast::AllFromOrganizationalUnitCastExists.new(cast)
        svc.add_parameter('castedSubjectUri', dto.casted_subject_uri)
        svc.add_parameter('recursive', dto.recursive)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('exists', organizational_unit_uri)
          return (res == 'true') || (res == '"true"')
        end
      end

    end
  end

end
