require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/friend/friend_attributes'
require 'uu/os/friend/friend_create'
require 'uu/os/friend/friend_set_attributes'


module UU
  module OS

    # Friend service
    module Friend
      extend self

      # Service path for the REST API
      PATH = 'ues/myterritory/person/UESFriend'

      # Returns attributes representing the friend specified by the +friend_uri+ parameter.
      # The command does not change the state of the friend in the system.
      #
      # @param [String, UU::OS::UESURI] friend_uri UESURI of the friend whose attributes are returned
      #
      # @return [FriendAttributes] DTO containing the friend attributes
      #
      # @example
      #   # Get attributes of a friend
      #   UU::OS::Friend.get_attributes('ues:TERRITORY:FRIEND')
      def get_attributes(friend_uri)
        svc = UU::OS::REST::RemoteClient.new(Friend)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', friend_uri)
          return Friend::FriendAttributes.new(res)
        end
      end

      # Command for setting attributes of a friend. The command can't modify references
      # to objects like location, competent role and so on.
      #
      # @param [String, UU::OS::UESURI] friend_uri UESURI of the friend whose attributes are to be changed
      # @param [FriendSetAttributes] criteria DTO containing new attributes of the friend
      # @option criteria [String] :first_name The new name of the role. If nil is set, the name is not modified. Note that the role name is automatically extended by the name of casted access roles names, when this attribute is not nil. Therefore the name attribute returned by the set_attributes command contains different string.
      # @option criteria [String] :surname The new code of the role. If nil is set, the code is not modified. The code must be unique in a given territory, otherwise the set_attributes command fails.
      # @option criteria [String] :code The new description of the role. If nil is set, the description is not modified.
      # @option criteria [String] :description The new description of the role. If nil is set, the description is not modified.
      # @option criteria [String, UU::OS::Artifact::SecurityLevel] :security_level The new description of the role. If nil is set, the description is not modified.
      # @option criteria [String, UU::OS::UESURI] :URI of the friend's icon. If nil is set, the icon is not modified. If nil URI ('ues:{[-1]}:{[-1]}:') is set, the friend's icon is removed and the friend uses icon from meta artifact. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      #
      # @return [UU::OS::UESURI] UESURI of the friend
      #
      # @example
      #   # Set a few attributes of a friend
      #   UU::OS::Friend.set_attributes('ues:TERRITORY:FRIEND',
      #       :surname => 'New Surname',
      #       :code => 'NEW_CODE',
      #       :security_level => UU::OS::Artifact::SecurityLevel::CONFIDENTIAL
      #   )
      def set_attributes(friend_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(Friend)
        payload = UU::OS::Friend::FriendSetAttributes.new(criteria).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', friend_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Creates a new friend. The command creates a new friend in a specified
      # personal territory. At the very least meta artifact must be specified
      # in {FriendCreate} DTO.
      #
      # @param [String, UU::OS::UESURI] personal_territory_uri UESURI of the personal territory where a new friend will be created
      # @param [FriendCreate] criteria DTO containing attributes of the new friend
      # @option criteria [String, UU::OS::UESURI] :meta_artifact_uri UESURI of the meta artifact. Only a meta artifact with the template class of Friend (not its descendants) can be used. If the rights does not permit to create a new friend from the meta artifact, the create command fails.
      # @option criteria [String] :universe_id Universe identifier of the friend. If it is not specified, the create command fails.
      # @option criteria [String] :first_name First name of the new friend. If it is not specified, the value is used from friend's personal role (but if it is not specified and the attribute surname is not nil, the command fails).
      # @option criteria [String] :surname Surname of the new friend. If it is not specified, the value is used from friend's personal role (but If it is not specified and the attribute name is not nil, the command fails).
      # @option criteria [String] :code Code of the new friend. If it is not specified, the code is generated automatically using the system sequence. The code must be unique in the given territory, otherwise the create command fails.
      # @option criteria [String] :description Description of the new friend. If it is not specified, tDescription from meta artifact is used if set, otherwise no description is set on the new friend.
      # @option criteria [String, UU::OS::Artifact::SecurityLevel] :security_level Security level of the new friend. If it is not specified, security level is taken from the meta artifact tSecurityLevel. If the meta artifact tSecurityLevel is higher than the user's security clearance, the create command fails.
      # @option criteria [String, UU::OS::UESURI] :icon_uri URI of the friend's icon. If it is not specified, the friend uses icon from meta artifact. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option criteria [String] :message Personal message for the friend. The message is used in friendship confirmation activity.
      #
      # @return [UU::OS::UESURI] UESURI of the new friend
      #
      # @example
      #   # Create a new friend (using only required attributes)
      #   UU::OS::Friend.create('ues:PERSONAL_TERRITORY:PERSONAL_TERRITORY',
      #       :meta_artifact_uri => 'ues:TERRITORY:META_ARTIFACT',
      #       :universe_id => '1234-56789-1'
      #   )
      #
      #   # Create a new friend (and set most of its attributes)
      #   UU::OS::Friend.create('ues:PERSONAL_TERRITORY:PERSONAL_TERRITORY',
      #       :meta_artifact_uri => 'ues:TERRITORY:META_ARTIFACT',
      #       :universe_id => '1234-56789-1',
      #       :first_name => 'John',
      #       :surname => 'Smith',
      #       :code => 'FRIEND_JOHN_SMITH',
      #       :description => 'Description of John Smith',
      #       :security_level => UU::OS::Artifact::SecurityLevel::NO_CONFIDENTIALITY,
      #       :icon_uri => 'ues:#{system}:#{ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{ART_322}',
      #       :message => 'I am your classmate'
      #   )
      def create(personal_territory_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(Friend)
        payload = UU::OS::Friend::FriendCreate.new(criteria).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', personal_territory_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Activates a friend. The friend will be again with access to the territory where is located.
      #
      # @param [String, UU::OS::UESURI] friend_uri UESURI of a friend that the access will be reactivated
      #
      # @example
      #   # Activate access of a friend
      #   UU::OS::Friend.activate_access('ues:TERRITORY:FRIEND')
      def activate_access(friend_uri)
        svc = UU::OS::REST::RemoteClient.new(Friend)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('activateAccess', friend_uri)
        end
      end

      # Deactivates a friend. The friend will be without access to the territory where is located.
      #
      # @param [String, UU::OS::UESURI] friend_uri UESURI of a friend that the access will be canceled
      #
      # @example
      #   # Deactivate access of a friend
      #   UU::OS::Friend.deactivate_access('ues:TERRITORY:FRIEND')
      def deactivate_access(friend_uri)
        svc = UU::OS::REST::RemoteClient.new(Friend)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('deactivateAccess', friend_uri)
        end
      end

    end

  end
end
