require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/image_catalog/image_catalog_get_image_list'
require 'uu/os/image_catalog/image_catalog_image_entry_attributes'

module UU
  module OS
    # ImageCatalog service.
    module ImageCatalog
      extend self

      # Service path
      PATH = 'ues/core/imagecatalog/UESImageCatalog'

      # Returns list of images from uuOS catalog. By default, the list is sorted by image names
      # (and codes in case names are equal). The list can be filtered using
      # "query" parameter in criteria using uuQuery 2.0 syntax.
      #
      # @param [String, UU::OS::UESURI] artifact_uri UESURI of an artifact for which images are to be listed.
      # @param [ImageCatalogGetImageList] criteria Object containing criteria for listing images. Available parameters for query are:
      #    {ImageCatalogImageEntryAttributes#name name} and {ImageCatalogImageEntryAttributes#code code}. See {https://plus4u.net/ues/sesm?SessFree=ues:UNI-BT:UU.OS%2FQUERY-2.0 uuQuery 2.0}.

      # @return [UU::OS::REST::ResultList<ImageCatalog::ImageCatalogGetImageList, ImageCatalog::ImageCatalogImageEntryAttributes>] List of Objects representing image entries.
      #
      # @example
      #   # get 3rd page of images using default ordering
      #   image_list = UU::OS::ImageCatalog.get_image_list('ues:TERRITORY:VUC_VIEW', :page_size => 50, :page_index => 2)
      #
      #   # get all icons whose codes start with ICON/ and use custom ordering
      #   image_list = UU::OS::ImageCatalog.get_image_list('ues:TERRITORY:MY_ART', :query => 'code LIKE ${var} ORDER BY code DESC',
      #       :query_parameters => { :var => 'ICON/%' })
      #
      def get_image_list(artifact_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(ImageCatalog)
        dto = ImageCatalog::ImageCatalogGetImageList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        if ((dto.query_parameters.kind_of?Hash) && (dto.query_parameters.size > 0))
          svc.add_parameter(:queryParameters, dto.query_parameters.to_json)
        end
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getImageList', artifact_uri)
          return UU::OS::REST::ResultList.new(ImageCatalog::ImageCatalogGetImageList, ImageCatalog::ImageCatalogImageEntryAttributes, res)
        end
      end
    end
  end
end