require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/role_search/role_search_query'
require 'uu/os/role_search/role_search_result_item'



module UU
  module OS
    module RoleSearch
      extend self

      # Service path
      PATH = 'ues/core/role/UESRoleSearch'
      
      # Performs a search for roles with specified criteria. The search is performed in
      # the location according to the aLocationUri, although it can be widened by using
      # includeSystemArea flag in the search criteria. 
      # 
      # Search query contains several search settings - query (what to search,
      # and order settings) and paging settings.
      #
      # **Recommended usage**
      #
      # - if location_uri is an organizational unit, use deepest organizational unit
      #   in the structure, so that the result set is restricted as much as possible.
      #   If the OU (its nesting level) is near the top of the territory, it might
      #   be better for performance to use the territory instead (if possible) - try measuring
      #   times in your case and decide based on that.
      #
      # @param [String, UU::OS::UESURI] location_uri The location to search in. Can be a territory or an organizational unit. 
      # @param [RoleSearchQuery] criteria DTO containing search criteria. Available parameters are: 
      # {RoleSearchResultItem#name name}, {RoleSearchResultItem#code code} and {RoleSearchResultItem#state_type stateType} : {Artifact::ArtifactStateType ArtifactStateType}. 
      #
      # See {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation}.
      #
      # @return [UU::OS::REST::ResultList<RoleSearch::RoleSearchQuery, RoleSearch::RoleSearchResultItem>] The list of found roles.
      def query(location_uri, criteria)
        svc = UU::OS::REST::RemoteClient.new(RoleSearch)
        dto = RoleSearch::RoleSearchQuery.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('query', location_uri)
          return UU::OS::REST::ResultList.new(RoleSearch::RoleSearchQuery, RoleSearch::RoleSearchResultItem, res)
        end
      end

    end
  end
end
