require 'uu/os/rest'
require 'uu/os/qos/qos_handler'
require 'uu/os/use_case_realization/use_case_realization_attributes'
require 'uu/os/use_case_realization/use_case_realization_create'
require 'uu/os/use_case_realization/use_case_realization_get_options'
require 'uu/os/use_case_realization/use_case_realization_get_use_case_list_item'
require 'uu/os/use_case_realization/use_case_realization_get_use_case_list'
require 'uu/os/use_case_realization/use_case_realization_set_attributes'
require 'uu/os/use_case_realization/use_case_realization_set_options'
require 'uu/os/use_case_realization/controller_engine'


module UU
  module OS   
      # Use Case Realization service.
      # @deprecated Use {UU::OS::VisualUseCase} instead.
      module UseCaseRealization
        extend self

        # Service path
        PATH = '/uu/os/UseCaseRealization'
        
        # The command creates a use case realization in a specified location. At the very least meta artifact must be specified in
        # {UseCaseRealizationCreate} DTO. Competent role for the new use case realization will be selected as the most suitable according
        # to specified container and meta artifact (executive/authorized role with connected interface), or can be also specified in
        # {UseCaseRealizationCreate} DTO.
        # 
        # @param [String, UU::OS::UESURI] uri UESURI of the MetaModelDictionary where the UCR should be created
        # @param [UseCaseRealizationCreate] dto DTO containing attributes for the new UCR
        #
        # @return [UU::OS::UESURI] UESURI of the created UCR
        def create(uri, dto = nil)
          svc = UU::OS::REST::RemoteClient.new(UseCaseRealization)
          payload = UU::OS::UseCaseRealization::UseCaseRealizationCreate.new(dto).to_json                   

          UU::OS::QoS::QoSHandler.auto_retry do
            res = svc.post('create', uri, payload)
            return UU::OS::UESURI.new(res)
          end
        end
        
        # The command deletes the specified use case realization from the system. The command fails when the specified use case realization is
        # referenced by some use case.
        #
        # @param [String, UU::OS::UESURI] uri UESURI of the UCR to be deleted
        def delete(uri)
          svc = UU::OS::REST::RemoteClient.new(UseCaseRealization)
          UU::OS::QoS::QoSHandler.auto_retry do
            svc.post('delete', uri)
          end
        end

        # This command returns attributes representing the use case realization specified by the useCaseRealizationUri parameter.
        #
        # @param [String, UU::OS::UESURI] use_case_realization_uri UESURI of the UCR
        #
        # @return [UseCaseRealizationAttributes] DTO containting the UCR attributes 
        def get_attributes(use_case_realization_uri)
          svc = UU::OS::REST::RemoteClient.new(UseCaseRealization)
          UU::OS::QoS::QoSHandler.auto_retry do
            res = svc.get('getAttributes', use_case_realization_uri)
            return UseCaseRealization::UseCaseRealizationAttributes.new(res)
          end
        end
        
        # The command for setting attributes of a specified use case realization. The command can't modify references to object like storage,
        # competent role and so on.
        # 
        # @param [String, UU::OS::UESURI] uri UESURI of the UCR
        # @param [UseCaseRealizationSetAttributes] dto DTO containing new attributes for given UCR
        #
        # @return [UU::OS::UESURI] UESURI of the changed UCR
        def set_attributes(uri, dto = nil)
          svc = UU::OS::REST::RemoteClient.new(UseCaseRealization)
          payload = UU::OS::UseCaseRealization::UseCaseRealizationSetAttributes.new(dto).to_json
  
          UU::OS::QoS::QoSHandler.auto_retry do
            res = svc.post('setAttributes', uri, payload)
            return UU::OS::UESURI.new(res)
          end
        end

        # Returns data of the first attachment on the UCR - supposed controller written in ruby
        #
        # @param [String, UU::OS::UESURI] use_case_realization_uri UESURI of the UCR
        #
        # @return [UU::OS::REST::BinaryValue] StreamHandler containing the data 
        def get_data(use_case_realization_uri)
          svc = UU::OS::REST::RemoteClient.new(UseCaseRealization)
          UU::OS::QoS::QoSHandler.auto_retry do
            res = svc.get_binary('getData', use_case_realization_uri)
            return UU::OS::REST::BinaryValue.new(res, true)
          end
        end
        
        
        # The command sets options of the specified use case realization.
        #
        # @param [String, UU::OS::UESURI] uri UseCaseRealization UESURI of the UCR
        # @param [UseCaseRealizationSetOptions] dto DTO containing new options for the UCR
        def set_options(uri, dto = nil)
          svc = UU::OS::REST::RemoteClient.new(UseCaseRealization)
          
          payload = UU::OS::UseCaseRealization::UseCaseRealizationSetOptions.new(dto).to_json                           
  
          UU::OS::QoS::QoSHandler.auto_retry do
            svc.post('setOptions', uri, payload)
          end
        end
  
        # The command returns options of the specified use case.
        # 
        # @param [String, UU::OS::UESURI] uri UESURI of the UCR
        # @param [UseCaseRealizationGetOptions] dto DTO containing details for the operation
        #
        # @return [UU::OS::REST::BinaryValue] StreamHandler containing the options 
        def get_options(uri, dto = nil)
          svc = UU::OS::REST::RemoteClient.new(UseCaseRealization)
          options = UU::OS::UseCaseRealization::UseCaseRealizationGetOptions.new(dto)
          svc.add_parameter('requestedMimeType', options.requested_mime_type)
          UU::OS::QoS::QoSHandler.auto_retry do
            return UU::OS::REST::BinaryValue.new(svc.get('getOptions', uri), true)
          end
        end
        
        # Returns list of the use cases that are connected to the specified use case realization. The list of returned use cases could be
        # filtered by the name or code. Returned list does not contain any objects filtered out by executed UESQuery. The list is sorted by the
        # name (and codes in case are equal) of an use case by default.
        #
        # @param [String, UU::OS::UESURI] uri UESURI of the UCR
        # @param [UseCaseRealizationGetUseCaseList] criteria DTO DTO containing criteria for listing connected use cases.
        #
        # See {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation}.
        #
        # @return [UU::OS::REST::ResultList<UseCaseRealization::UseCaseRealizationGetUseCaseList, UseCaseRealization::UseCaseRealizationGetUseCaseListItem>] List of the connected usecases
        def get_use_case_list(uri, criteria=nil)
          svc = UU::OS::REST::RemoteClient.new(UseCaseRealization)
          dto = UseCaseRealization::UseCaseRealizationGetUseCaseList.new(criteria)
          svc.add_parameter('pageIndex', dto.page_index)
          svc.add_parameter('pageSize', dto.page_size)
          svc.add_parameter('query', dto.query)
          UU::OS::QoS::QoSHandler.auto_retry do
            res = svc.get('getUseCaseList', uri)
            return UU::OS::REST::ResultList.new(UseCaseRealization::UseCaseRealizationGetUseCaseList, UseCaseRealization::UseCaseRealizationGetUseCaseListItem, res)
          end
        end

      end

  end
end
